{% load thumbnail %}
{% spaceless %}
<a href="{{ sponsor.external_url }}">
    {% if dimensions %}
        <img id="sponsor_{{ sponsor.pk }}" src="{% thumbnail sponsor.logo dimensions %}" alt="{{ sponsor.name }}" />
    {% else %}
        <img id="sponsor_{{ sponsor.pk }}" src="{% thumbnail sponsor.logo '150x150' %}" alt="{{ sponsor.name }}" />
    {% endif %}
</a>
{% endspaceless %}
