{% extends "site_base.html" %}

{% load i18n %}
{% load bootstrap_tags %}
{% load boxes_tags %}
{% load cache %}
{% load sitetree %}

{% block head_title %}Conference Schedule{% endblock %}

{% block body_class %}full{% endblock %}

{% block right %}
{% endblock %}

{% block body_outer %}
    <div class="row">
        <div class="span12">
            <div class="page-head">
                <h1>{{ schedule.section }} Schedule</h1>
                {% block breadcrumbs %}{% sitetree_breadcrumbs from "main" %}{% endblock %}
            </div>
        </div>
        <div class="span12">
            {% box "schedule_top_"|add:schedule.section.name|slugify %}
            
            {% cache 600 "schedule-table" schedule.section %}
                {% for timetable in days %}
                    <h3>{{ timetable.day.date }}</h3>
                    {% include "schedule/_grid.html" %}
                {% endfor %}
            {% endcache %}
            
            {% box "schedule_bottom" %}
        </div>
    </div>
{% endblock %}
