{% extends "site_base.html" %}

{% load bootstrap_tags %}
{% load i18n %}
{% load boxes_tags %}

{% block head_title %}{% trans "Apply to be a Sponsor" %}{% endblock %}

{% block body_class %}sponsorships{% endblock %}

{% block body %}

    {% box "sponsorship-apply" %}

    <form method="POST" action="" class="form-horizontal">
        {% csrf_token %}
        <legend>{% trans "Apply to Be a Sponsor" %}</legend>
        {{ form|as_bootstrap }}
        <div class="form-actions">
            <input class="btn btn-primary" type="submit" value="Apply" />
            <a class="btn" href="{% url dashboard %}">Cancel</a>
            <p class="help-block">
                <small>By submitting this sponsor application you are agreeing to the <a href="{% url cms_page "sponsor/terms/" %}" target="_blank">terms and conditions</a>.</small>
            </p>
        </div>
    </form>

{% endblock %}
