{% extends "site_base.html" %}

{% load bootstrap_tags %}

{% block head_title %}Upload File{% endblock %}

{% block body_outer %}
    <div class="row">
        <div class="span12">
            <h1>Upload File</h1>
            <form method="POST" action="{% url file_create %}" enctype="multipart/form-data">
                {% csrf_token %}
                {{ form|as_bootstrap }}
                <div class="form-actions">
                    <button class="btn btn-primary" type="submit">Upload</button>
                </div>
            </form>
        </div>
    </div>
{% endblock %}